/*
1. Create a staging table for the devices with installed updates
2. Create a staging table for all software updates
3. Create a staging table for Device IDs with Installed Updates
4. Parse JSON results and insert values into devices with installed updates staging table
5. Parse JSON results and insert values into all software updates staging table
6. Join Software Updates to Installed Update devices and create a comma delimited list of resource ids
7. Append this list as a node to the GetAllSoftwareUpdates JSON results
*/

IF EXISTS (
		SELECT *
		FROM sys.objects
		WHERE object_id = OBJECT_ID(N'[dbo].[stgDevicesWithInstalledUpdates]')
			AND type IN (N'U')
		)
	DROP TABLE [dbo].[stgDevicesWithInstalledUpdates]
GO

CREATE TABLE [dbo].[stgDevicesWithInstalledUpdates] (
	[snapshotconfigid] [int] NOT NULL
	,[ResourceID] [nvarchar](4000) NULL
	,[CI_ID] [nvarchar](4000) NULL
	) ON [PRIMARY]
GO

IF EXISTS (
		SELECT *
		FROM sys.objects
		WHERE object_id = OBJECT_ID(N'[dbo].[stgSoftwareUpdates]')
			AND type IN (N'U')
		)
	DROP TABLE [dbo].[stgSoftwareUpdates]
GO

CREATE TABLE [dbo].[stgSoftwareUpdates] (
	[SnapshotConfigID] [int] NOT NULL
	,[CI_ID] [nvarchar](4000) NULL
	,[ResultsId] [bigint] NOT NULL
	) ON [PRIMARY]
GO

IF EXISTS (
		SELECT *
		FROM sys.objects
		WHERE object_id = OBJECT_ID(N'[dbo].[stgDevicesIDWithInstallUpdates]')
			AND type IN (N'U')
		)
	DROP TABLE [dbo].[stgDevicesIDWithInstallUpdates]
GO

CREATE TABLE [dbo].[stgDevicesIDWithInstallUpdates] (
	[ResultsId] [bigint] NOT NULL
	,[DevicesWithInstalledUpdates] [nvarchar](max) NULL
	) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO

/*!!!!!!!!!!!!!!!!!!!!!*/
DECLARE @SnapshotID int = 15 -- <<<< Set this to the number in the URL of the snapshot you want to migrate
/*!!!!!!!!!!!!!!!!!!!!!*/


INSERT INTO [dbo].[stgDevicesWithInstalledUpdates] (
	[snapshotconfigid]
	,[ResourceID]
	,[CI_ID]
	)
SELECT DISTINCT sc.snapshotconfigid
	,JSON_Value(aer.SerializedValue, '$.ResourceID') AS ResourceID
	,JSON_Value(aer.SerializedValue, '$.CI_ID') AS CI_ID
FROM rct.SnapshotConfigs sc
INNER JOIN rct.ActionExecutionGroups aeg ON aeg.SnapshotConfigID = sc.SnapshotConfigID
INNER JOIN rct.ActionExecutionSet aes ON aes.ActionExecutionGroupID = aeg.id
INNER JOIN dbo.ActionExecutions ae ON aes.ActionExecutionID = ae.id
INNER JOIN dbo.ActionExecutionResults aer ON aer.ActionExecutionID = ae.id
INNER JOIN dbo.RecastActions ra ON ra.Id = ae.ActionID
INNER JOIN [stgDeviceCollectionMembers] dc ON dc.resourceid = JSON_Value(aer.SerializedValue, '$.ResourceID')
	AND dc.SnapshotConfigID = sc.SnapshotConfigID
WHERE sc.type = 4
	AND ra.[name] = 'GetInstalledSoftwareUpdates'
	AND aer.SerializedValue IS NOT NULL
	AND aeg.ID = @SnapshotID


INSERT INTO [dbo].[stgSoftwareUpdates] (
	[SnapshotConfigID]
	,[CI_ID]
	,[ResultsId]
	)
SELECT DISTINCT sc.SnapshotConfigID
	,JSON_Value(aer.SerializedValue, '$.Update.CI_ID') AS CI_ID
	,aer.id AS [ResultsId]
FROM rct.SnapshotConfigs sc
INNER JOIN rct.ActionExecutionGroups aeg ON aeg.SnapshotConfigID = sc.SnapshotConfigID
INNER JOIN rct.ActionExecutionSet aes ON aes.ActionExecutionGroupID = aeg.id
INNER JOIN dbo.ActionExecutions ae ON aes.ActionExecutionID = ae.id
INNER JOIN dbo.ActionExecutionResults aer ON aer.ActionExecutionID = ae.id
INNER JOIN dbo.RecastActions ra ON ra.Id = ae.ActionID
WHERE sc.type = 4
	AND ra.[name] = 'GetAllSoftwareUpdates'
	AND aer.SerializedValue IS NOT NULL
	AND aeg.ID = @SnapshotID

INSERT INTO [dbo].[stgDevicesIDWithInstallUpdates] (
	[ResultsId]
	,[DevicesWithInstalledUpdates]
	)
SELECT su.ResultsId
	,'[' + STUFF((
			SELECT ',' + diu2.ResourceID
			FROM [stgSoftwareUpdates] su2
			INNER JOIN [stgDevicesWithInstalledUpdates] diu2 ON diu2.SnapshotConfigID = su2.SnapshotConfigID
				AND diu2.ci_id = su2.CI_ID
			WHERE su2.ResultsId = su.ResultsId
			FOR XML PATH('')
			), 1, 1, '') + ']' AS DevicesWithInstalledUpdates
FROM [stgSoftwareUpdates] su
INNER JOIN [stgDevicesWithInstalledUpdates] diu ON diu.SnapshotConfigID = su.SnapshotConfigID
	AND diu.ci_id = su.CI_ID
GROUP BY su.ResultsId

UPDATE aer
SET SerializedValue = REPLACE(REPLACE(JSON_MODIFY(aer.SerializedValue, 'append $.DeviceResourceIdsWithUpdateInstalled', JSON_QUERY(DevicesWithInstalledUpdates)), '[[', '['), ']]', ']') --AS 'Updated SerializedValue'
FROM dbo.ActionExecutionResults aer
INNER JOIN dbo.stgDevicesIDWithInstallUpdates iu ON iu.ResultsId = aer.id